==> interzone-1.0.17.tar.gz: lib/moderator/OutgoingMessageCache.dart <==
import 'dart:math';
import 'dart:typed_data';
import 'package:interzone/disk.dart';
import 'OutgoingMessage.dart';

class OutgoingMessageCache {
  Map<int, OutgoingMessage> _data = {};
  Set<int> _sent = {};
  Set<int> _pending = {};
  interzoneDisk? _io;
  String _ioSiteName = '';
  init(interzoneDisk io, String siteName) async {
    _io = io;
    _ioSiteName = siteName;
    /*OutgoingMessage test = OutgoingMessage(
        0, 'nick', 'text', 'root', 'pp', 0, Uint8List(0), 'cid');
    Uint8List tb = test.binary;
    var y = OutgoingMessage.fromBinary(tb);
    print('r');*/
    //read pending items on the disk queue
    final ri = await _io?.readUint64List(_ioSiteName + 'ogmDq');
    //TODO lazy pull when necessary
    if (ri != null && ri.isNotEmpty) {
      _diskQueued = ri.toSet();
      for (var i in ri) {
        final en = await pullFromDiskQueue(i);
        if (en != null) {
          _data[i] = en;
        }
      }
    }
  }

  Future<OutgoingMessage?> pickPendingItem(int t) async {
    if (_sent.contains(t) || _pending.contains(t)) return null;

    if (_data[t] != null) {
      return _data[t];
    }

    return await pullFromDiskQueue(t);
  }

  OutgoingMessage? get pendingItem {
    if (_data.isEmpty) return null;
    final p = _data.keys
        .where((element) =>
            !_sent.contains(element) && !_pending.contains(element))
        .toList();
    if (p.isEmpty) return null;
    return _data[p.first];
  }

  void holdPendingItem(int i) {
    _pending.add(i);
    _io?.commitUint64List(_pending.toList(), _ioSiteName + 'ogmPending');
  }

  void releasePendingItem(int i) {
    _pending.remove(i);
    _io?.commitUint64List(_pending.toList(), _ioSiteName + 'ogmPending');
  }

  Set<int> _diskQueued = {};

  Future<bool> pushToDiskQueue(int i) async {
    if (_diskQueued.contains(i)) return true;
    final b = _data[i]?.binary;
    if (b == null) return false;
    final pd =
        await _io?.commitUint8List(b, _ioSiteName + 'ogmDq' + i.toString());
    if (pd != null && pd) {
      _diskQueued.add(i);
      final pdd = await _io?.commitUint64List(
          _diskQueued.toList(), _ioSiteName + 'ogmDq');
      return true;
    }
    return false;
  }

  Future<OutgoingMessage?> pullFromDiskQueue(int i) async {
    if (_data[i] != null) return _data[i]!;
    final pd = await _io?.readUint8List(_ioSiteName + 'ogmDq' + i.toString());
    if (pd != null && pd.isNotEmpty) {
      try {
        final ie = OutgoingMessage.fromBinary(pd);
        _data[i] = ie;
        return ie;
      } catch (e) {
        print('pullFromDiskQueue item exploded');
        _io?.deleteUint8List(_ioSiteName + 'ogmDq' + i.toString());
      }
    }
    return null;
  }

  Future<bool> deleteFromDiskQueue(int i) async {
    _diskQueued.remove(i);
    final pd = await _io?.deleteUint8List(_ioSiteName + 'ogmDq' + i.toString());
    if (pd != null && pd) {
      return true;
    }
    return false;
  }

  OutgoingMessage? addPost(nick, avatar, text, rootKey, ppKey, timestamp,
      attachement, cid, attachmentType) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(0, nick, text, rootKey, ppKey, timestamp,
        attachement, cid, attachmentType);
    return _data[timestamp];
  }

  OutgoingMessage? addReply(nick, avatar, text, rootKey, ppKey, timestamp,
      attachement, cid, attachementType) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(1, nick, text, rootKey, ppKey, timestamp,
        attachement, cid, attachementType);
    return _data[timestamp];
  }

  OutgoingMessage? addDrop(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        3, '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addMute(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        4, '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addShare(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        5, '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addLike(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        6, '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addunLike(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        7, '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  bool commitSent(timestamp) {
    _sent.add(timestamp);
    deleteFromDiskQueue(timestamp);
    releasePendingItem(timestamp);
    _data.remove(timestamp);
    return true;
  }

  bool setCID(timestamp, cid) {
    if (_data[timestamp] == null) return false;
    _data[timestamp]!.cid = cid;
    return true;
  }
}

==> interzone-1.0.19.tar.gz: lib/moderator/OutgoingMessageCache.dart <==
import 'dart:math';
import 'dart:typed_data';
import 'package:interzone/disk.dart';
import 'OutgoingMessage.dart';

class OutgoingMessageCache {
  Map<int, OutgoingMessage> _data = {};
  Set<int> _sent = {};
  Set<int> _pending = {};
  interzoneDisk? _io;
  String _ioSiteName = '';
  init(interzoneDisk io, String siteName) async {
    _io = io;
    _ioSiteName = siteName;
    /*OutgoingMessage test = OutgoingMessage(
        0, 'nick', 'text', 'root', 'pp', 0, Uint8List(0), 'cid');
    Uint8List tb = test.binary;
    var y = OutgoingMessage.fromBinary(tb);
    print('r');*/
    //read pending items on the disk queue
    final ri = await _io?.readUint64List(_ioSiteName + 'ogmDq');
    //TODO lazy pull when necessary
    if (ri != null && ri.isNotEmpty) {
      _diskQueued = ri.toSet();
      for (var i in ri) {
        final en = await pullFromDiskQueue(i);
        if (en != null) {
          _data[i] = en;
        }
      }
    }
  }

  Future<OutgoingMessage?> pickPendingItem(int t) async {
    if (_sent.contains(t) || _pending.contains(t)) return null;

    if (_data[t] != null) {
      return _data[t];
    }

    return await pullFromDiskQueue(t);
  }

  OutgoingMessage? get pendingItem {
    if (_data.isEmpty) return null;
    final p = _data.keys
        .where((element) =>
            !_sent.contains(element) && !_pending.contains(element))
        .toList();
    if (p.isEmpty) return null;
    return _data[p.first];
  }

  void holdPendingItem(int i) {
    _pending.add(i);
    _io?.commitUint64List(_pending.toList(), _ioSiteName + 'ogmPending');
  }

  void releasePendingItem(int i) {
    _pending.remove(i);
    _io?.commitUint64List(_pending.toList(), _ioSiteName + 'ogmPending');
  }

  Set<int> _diskQueued = {};

  Future<bool> pushToDiskQueue(int i) async {
    if (_diskQueued.contains(i)) return true;
    final b = _data[i]?.binary;
    if (b == null) return false;
    final pd =
        await _io?.commitUint8List(b, _ioSiteName + 'ogmDq' + i.toString());
    if (pd != null && pd) {
      _diskQueued.add(i);
      final pdd = await _io?.commitUint64List(
          _diskQueued.toList(), _ioSiteName + 'ogmDq');
      return true;
    }
    return false;
  }

  Future<OutgoingMessage?> pullFromDiskQueue(int i) async {
    if (_data[i] != null) return _data[i]!;
    final pd = await _io?.readUint8List(_ioSiteName + 'ogmDq' + i.toString());
    if (pd != null && pd.isNotEmpty) {
      try {
        final ie = OutgoingMessage.fromBinary(pd);
        _data[i] = ie;
        return ie;
      } catch (e) {
        print('pullFromDiskQueue item exploded');
        _io?.deleteUint8List(_ioSiteName + 'ogmDq' + i.toString());
      }
    }
    return null;
  }

  Future<bool> deleteFromDiskQueue(int i) async {
    _diskQueued.remove(i);
    final pd = await _io?.deleteUint8List(_ioSiteName + 'ogmDq' + i.toString());
    if (pd != null && pd) {
      return true;
    }
    return false;
  }

  OutgoingMessage? addPost(nick, avatar, text, rootKey, ppKey, timestamp,
      attachement, cid, attachmentType) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(0, nick, avatar, text, rootKey, ppKey,
        timestamp, attachement, cid, attachmentType);
    return _data[timestamp];
  }

  OutgoingMessage? addReply(nick, avatar, text, rootKey, ppKey, timestamp,
      attachement, cid, attachementType) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(1, nick, avatar, text, rootKey, ppKey,
        timestamp, attachement, cid, attachementType);
    return _data[timestamp];
  }

  OutgoingMessage? addDrop(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        3, '', '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addMute(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        4, '', '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addShare(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        5, '', '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addLike(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        6, '', '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  OutgoingMessage? addunLike(String rootKey, int timestamp) {
    if (_data[timestamp] != null) return null;
    if (_sent.contains(timestamp)) return null;

    _data[timestamp] = OutgoingMessage(
        7, '', '', '', rootKey, '', timestamp, Uint8List(0), '', '');
    return _data[timestamp];
  }

  bool commitSent(timestamp) {
    _sent.add(timestamp);
    deleteFromDiskQueue(timestamp);
    releasePendingItem(timestamp);
    _data.remove(timestamp);
    return true;
  }

  bool setCID(timestamp, cid) {
    if (_data[timestamp] == null) return false;
    _data[timestamp]!.cid = cid;
    return true;
  }
}
